@@ -376,16 +376,18 @@ Status CompactionServiceCompactionJob::Run() {
   // Build Output
   compaction_result_->output_level = compact_->compaction->output_level();
   compaction_result_->output_path = output_path_;
-  for (const auto& output_file : sub_compact->GetOutputs()) {
-    auto& meta = output_file.meta;
-    compaction_result_->output_files.emplace_back(
-        MakeTableFileName(meta.fd.GetNumber()), meta.fd.smallest_seqno,
-        meta.fd.largest_seqno, meta.smallest.Encode().ToString(),
-        meta.largest.Encode().ToString(), meta.oldest_ancester_time,
-        meta.file_creation_time, meta.epoch_number, meta.file_checksum,
-        meta.file_checksum_func_name, output_file.validator.GetHash(),
-        meta.marked_for_compaction, meta.unique_id,
-        output_file.table_properties);
+  if (status.ok()) {
+    for (const auto& output_file : sub_compact->GetOutputs()) {
+      auto& meta = output_file.meta;
+      compaction_result_->output_files.emplace_back(
+          MakeTableFileName(meta.fd.GetNumber()), meta.fd.smallest_seqno,
+          meta.fd.largest_seqno, meta.smallest.Encode().ToString(),
+          meta.largest.Encode().ToString(), meta.oldest_ancester_time,
+          meta.file_creation_time, meta.epoch_number, meta.file_checksum,
+          meta.file_checksum_func_name, output_file.validator.GetHash(),
+          meta.marked_for_compaction, meta.unique_id,
+          *output_file.table_properties);
+    }
   }
 
   TEST_SYNC_POINT_CALLBACK("CompactionServiceCompactionJob::Run:0",
@@ -477,159 +479,6 @@ static std::unordered_map<std::string, OptionTypeInfo> cs_input_type_info = {
       OptionType::kUInt64T, OptionVerificationType::kNormal,
       OptionTypeFlags::kNone}},
 };
-static std::unordered_map<std::string, OptionTypeInfo>
-    table_properties_type_info = {
-        {"orig_file_number",
-         {offsetof(struct TableProperties, orig_file_number),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"data_size",
-         {offsetof(struct TableProperties, data_size), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"index_size",
-         {offsetof(struct TableProperties, index_size), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"index_partitions",
-         {offsetof(struct TableProperties, index_partitions),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"top_level_index_size",
-         {offsetof(struct TableProperties, top_level_index_size),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"index_key_is_user_key",
-         {offsetof(struct TableProperties, index_key_is_user_key),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"index_value_is_delta_encoded",
-         {offsetof(struct TableProperties, index_value_is_delta_encoded),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"filter_size",
-         {offsetof(struct TableProperties, filter_size), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"raw_key_size",
-         {offsetof(struct TableProperties, raw_key_size), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"raw_value_size",
-         {offsetof(struct TableProperties, raw_value_size),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"num_data_blocks",
-         {offsetof(struct TableProperties, num_data_blocks),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"num_entries",
-         {offsetof(struct TableProperties, num_entries), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"num_filter_entries",
-         {offsetof(struct TableProperties, num_filter_entries),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"num_deletions",
-         {offsetof(struct TableProperties, num_deletions), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"num_merge_operands",
-         {offsetof(struct TableProperties, num_merge_operands),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"num_range_deletions",
-         {offsetof(struct TableProperties, num_range_deletions),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"format_version",
-         {offsetof(struct TableProperties, format_version),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"fixed_key_len",
-         {offsetof(struct TableProperties, fixed_key_len), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"column_family_id",
-         {offsetof(struct TableProperties, column_family_id),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"creation_time",
-         {offsetof(struct TableProperties, creation_time), OptionType::kUInt64T,
-          OptionVerificationType::kNormal, OptionTypeFlags::kNone}},
-        {"oldest_key_time",
-         {offsetof(struct TableProperties, oldest_key_time),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"file_creation_time",
-         {offsetof(struct TableProperties, file_creation_time),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"slow_compression_estimated_data_size",
-         {offsetof(struct TableProperties,
-                   slow_compression_estimated_data_size),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"fast_compression_estimated_data_size",
-         {offsetof(struct TableProperties,
-                   fast_compression_estimated_data_size),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"external_sst_file_global_seqno_offset",
-         {offsetof(struct TableProperties,
-                   external_sst_file_global_seqno_offset),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"tail_start_offset",
-         {offsetof(struct TableProperties, tail_start_offset),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"user_defined_timestamps_persisted",
-         {offsetof(struct TableProperties, user_defined_timestamps_persisted),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"key_largest_seqno",
-         {offsetof(struct TableProperties, key_largest_seqno),
-          OptionType::kUInt64T, OptionVerificationType::kNormal,
-          OptionTypeFlags::kNone}},
-        {"db_id",
-         {offsetof(struct TableProperties, db_id), OptionType::kEncodedString}},
-        {"db_session_id",
-         {offsetof(struct TableProperties, db_session_id),
-          OptionType::kEncodedString}},
-        {"db_host_id",
-         {offsetof(struct TableProperties, db_host_id),
-          OptionType::kEncodedString}},
-        {"column_family_name",
-         {offsetof(struct TableProperties, column_family_name),
-          OptionType::kEncodedString}},
-        {"filter_policy_name",
-         {offsetof(struct TableProperties, filter_policy_name),
-          OptionType::kEncodedString}},
-        {"comparator_name",
-         {offsetof(struct TableProperties, comparator_name),
-          OptionType::kEncodedString}},
-        {"merge_operator_name",
-         {offsetof(struct TableProperties, merge_operator_name),
-          OptionType::kEncodedString}},
-        {"prefix_extractor_name",
-         {offsetof(struct TableProperties, prefix_extractor_name),
-          OptionType::kEncodedString}},
-        {"property_collectors_names",
-         {offsetof(struct TableProperties, property_collectors_names),
-          OptionType::kEncodedString}},
-        {"compression_name",
-         {offsetof(struct TableProperties, compression_name),
-          OptionType::kEncodedString}},
-        {"compression_options",
-         {offsetof(struct TableProperties, compression_options),
-          OptionType::kEncodedString}},
-        {"seqno_to_time_mapping",
-         {offsetof(struct TableProperties, seqno_to_time_mapping),
-          OptionType::kEncodedString}},
-        {"user_collected_properties",
-         OptionTypeInfo::StringMap(
-             offsetof(struct TableProperties, user_collected_properties),
-             OptionVerificationType::kNormal, OptionTypeFlags::kNone)},
-        {"readable_properties",
-         OptionTypeInfo::StringMap(
-             offsetof(struct TableProperties, readable_properties),
-             OptionVerificationType::kNormal, OptionTypeFlags::kNone)},
-};
 
 static std::unordered_map<std::string, OptionTypeInfo>
     cs_output_file_type_info = {
@@ -687,11 +536,29 @@ static std::unordered_map<std::string, OptionTypeInfo>
              OptionVerificationType::kNormal, OptionTypeFlags::kNone,
              {0, OptionType::kUInt64T})},
         {"table_properties",
-         OptionTypeInfo::Struct(
-             "table_properties", &table_properties_type_info,
-             offsetof(struct CompactionServiceOutputFile, table_properties),
-             OptionVerificationType::kNormal, OptionTypeFlags::kNone)},
-};
+         {offsetof(struct CompactionServiceOutputFile, table_properties),
+          OptionType::kStruct, OptionVerificationType::kNormal,
+          OptionTypeFlags::kNone,
+          [](const ConfigOptions& opts, const std::string& /*name*/,
+             const std::string& value, void* addr) {
+            auto table_properties = static_cast<TableProperties*>(addr);
+            return TableProperties::Parse(opts, value, table_properties);
+          },
+          [](const ConfigOptions& opts, const std::string& /*name*/,
+             const void* addr, std::string* value) {
+            const auto table_properties =
+                static_cast<const TableProperties*>(addr);
+            std::string result;
+            auto status = table_properties->Serialize(opts, &result);
+            *value = "{" + result + "}";
+            return status;
+          },
+          [](const ConfigOptions& opts, const std::string& /*name*/,
+             const void* addr1, const void* addr2, std::string* mismatch) {
+            const auto this_one = static_cast<const TableProperties*>(addr1);
+            const auto that_one = static_cast<const TableProperties*>(addr2);
+            return this_one->AreEqual(opts, that_one, mismatch);
+          }}}};
 
 static std::unordered_map<std::string, OptionTypeInfo>
     compaction_job_stats_type_info = {
