@@ -71,22 +71,46 @@ class DbStressListener : public EventListener {
     VerifyFilePath(info.file_path);
     // pretending doing some work here
     RandomSleep();
-    if (FLAGS_read_fault_one_in) {
-      (void)fault_fs_guard->GetAndResetErrorCount();
-      fault_fs_guard->DisableErrorInjection();
+    if (fault_fs_guard) {
+      fault_fs_guard->DisableAllThreadLocalErrorInjection();
     }
   }
 
   void OnFlushBegin(DB* /*db*/,
                     const FlushJobInfo& /*flush_job_info*/) override {
     RandomSleep();
-    if (FLAGS_read_fault_one_in) {
-      // Hardcoded to inject retryable error as a non-retryable error would put
-      // the DB in read-only mode and then it would crash on the next write.
-      fault_fs_guard->SetThreadLocalReadErrorContext(
-          static_cast<uint32_t>(FLAGS_seed), FLAGS_read_fault_one_in,
-          true /* retryable */);
-      fault_fs_guard->EnableErrorInjection();
+    if (fault_fs_guard) {
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kRead, static_cast<uint32_t>(FLAGS_seed),
+          FLAGS_read_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kRead);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kWrite, static_cast<uint32_t>(FLAGS_seed),
+          FLAGS_write_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kWrite);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kMetadataRead,
+          static_cast<uint32_t>(FLAGS_seed), FLAGS_metadata_read_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kMetadataRead);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kMetadataWrite,
+          static_cast<uint32_t>(FLAGS_seed), FLAGS_metadata_write_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kMetadataWrite);
     }
   }
 
@@ -112,20 +136,44 @@ class DbStressListener : public EventListener {
   }
 
   void OnSubcompactionBegin(const SubcompactionJobInfo& /* si */) override {
-    if (FLAGS_read_fault_one_in) {
-      // Hardcoded to inject retryable error as a non-retryable error would put
-      // the DB in read-only mode and then it would crash on the next write.
-      fault_fs_guard->SetThreadLocalReadErrorContext(
-          static_cast<uint32_t>(FLAGS_seed), FLAGS_read_fault_one_in,
-          true /* retryable */);
-      fault_fs_guard->EnableErrorInjection();
+    if (fault_fs_guard) {
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kRead, static_cast<uint32_t>(FLAGS_seed),
+          FLAGS_read_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kRead);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kWrite, static_cast<uint32_t>(FLAGS_seed),
+          FLAGS_write_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kWrite);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kMetadataRead,
+          static_cast<uint32_t>(FLAGS_seed), FLAGS_metadata_read_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kMetadataRead);
+
+      fault_fs_guard->SetThreadLocalErrorContext(
+          FaultInjectionIOType::kMetadataWrite,
+          static_cast<uint32_t>(FLAGS_seed), FLAGS_metadata_write_fault_one_in,
+          FLAGS_inject_error_severity == 1 /* retryable */,
+          FLAGS_inject_error_severity == 2 /* has_data_loss*/);
+      fault_fs_guard->EnableThreadLocalErrorInjection(
+          FaultInjectionIOType::kMetadataWrite);
     }
   }
 
   void OnSubcompactionCompleted(const SubcompactionJobInfo& /* si */) override {
-    if (FLAGS_read_fault_one_in) {
-      (void)fault_fs_guard->GetAndResetErrorCount();
-      fault_fs_guard->DisableErrorInjection();
+    if (fault_fs_guard) {
+      fault_fs_guard->DisableAllThreadLocalErrorInjection();
     }
   }
 
@@ -211,10 +259,22 @@ class DbStressListener : public EventListener {
                             Status /* bg_error */,
                             bool* /* auto_recovery */) override {
     RandomSleep();
+    if (FLAGS_error_recovery_with_no_fault_injection && fault_fs_guard) {
+      fault_fs_guard->DisableAllThreadLocalErrorInjection();
+      // TODO(hx235): only exempt the flush thread during error recovery instead
+      // of all the flush threads from error injection
+      fault_fs_guard->SetIOActivtiesExcludedFromFaultInjection(
+          {Env::IOActivity::kFlush});
+    }
   }
 
-  void OnErrorRecoveryCompleted(Status /* old_bg_error */) override {
+  void OnErrorRecoveryEnd(
+      const BackgroundErrorRecoveryInfo& /*info*/) override {
     RandomSleep();
+    if (FLAGS_error_recovery_with_no_fault_injection && fault_fs_guard) {
+      fault_fs_guard->EnableAllThreadLocalErrorInjection();
+      fault_fs_guard->SetIOActivtiesExcludedFromFaultInjection({});
+    }
   }
 
  protected:
