@@ -4,13 +4,15 @@
 //  (found in the LICENSE.Apache file in the root directory).
 //
 // This file implements the "bridge" between Java and C++ for
-// rocksdb::CompactionFilterFactory.
+// ROCKSDB_NAMESPACE::CompactionFilterFactory.
 
 #include <jni.h>
+
 #include <memory>
 
 #include "include/org_rocksdb_AbstractCompactionFilterFactory.h"
 #include "rocksjni/compaction_filter_factory_jnicallback.h"
+#include "rocksjni/cplusplus_to_java_convert.h"
 
 /*
  * Class:     org_rocksdb_AbstractCompactionFilterFactory
@@ -19,10 +21,11 @@
  */
 jlong Java_org_rocksdb_AbstractCompactionFilterFactory_createNewCompactionFilterFactory0(
     JNIEnv* env, jobject jobj) {
-  auto* cff = new rocksdb::CompactionFilterFactoryJniCallback(env, jobj);
-  auto* ptr_sptr_cff =
-      new std::shared_ptr<rocksdb::CompactionFilterFactoryJniCallback>(cff);
-  return reinterpret_cast<jlong>(ptr_sptr_cff);
+  auto* cff =
+      new ROCKSDB_NAMESPACE::CompactionFilterFactoryJniCallback(env, jobj);
+  auto* ptr_sptr_cff = new std::shared_ptr<
+      ROCKSDB_NAMESPACE::CompactionFilterFactoryJniCallback>(cff);
+  return GET_CPLUSPLUS_POINTER(ptr_sptr_cff);
 }
 
 /*
@@ -31,9 +34,9 @@ jlong Java_org_rocksdb_AbstractCompactionFilterFactory_createNewCompactionFilter
  * Signature: (J)V
  */
 void Java_org_rocksdb_AbstractCompactionFilterFactory_disposeInternal(
-    JNIEnv* /*env*/, jobject /*jobj*/, jlong jhandle) {
+    JNIEnv*, jclass, jlong jhandle) {
   auto* ptr_sptr_cff = reinterpret_cast<
-      std::shared_ptr<rocksdb::CompactionFilterFactoryJniCallback>*>(jhandle);
+      std::shared_ptr<ROCKSDB_NAMESPACE::CompactionFilterFactoryJniCallback>*>(
+      jhandle);
   delete ptr_sptr_cff;
-  // @lint-ignore TXT4 T25377293 Grandfathered in
 }
