@@ -224,37 +224,15 @@ LoggerJniCallback::~LoggerJniCallback() {
 
 /*
  * Class:     org_rocksdb_Logger
- * Method:    createNewLoggerOptions
+ * Method:    newLogger
  * Signature: (J)J
  */
-jlong Java_org_rocksdb_Logger_createNewLoggerOptions(JNIEnv* env, jobject jobj,
-                                                     jlong joptions) {
+jlong Java_org_rocksdb_Logger_newLogger(JNIEnv* env, jobject jobj,
+                                        jlong jlog_level) {
   auto* sptr_logger = new std::shared_ptr<ROCKSDB_NAMESPACE::LoggerJniCallback>(
       new ROCKSDB_NAMESPACE::LoggerJniCallback(env, jobj));
-
-  // set log level
-  auto* options = reinterpret_cast<ROCKSDB_NAMESPACE::Options*>(joptions);
-  sptr_logger->get()->SetInfoLogLevel(options->info_log_level);
-
-  return GET_CPLUSPLUS_POINTER(sptr_logger);
-}
-
-/*
- * Class:     org_rocksdb_Logger
- * Method:    createNewLoggerDbOptions
- * Signature: (J)J
- */
-jlong Java_org_rocksdb_Logger_createNewLoggerDbOptions(JNIEnv* env,
-                                                       jobject jobj,
-                                                       jlong jdb_options) {
-  auto* sptr_logger = new std::shared_ptr<ROCKSDB_NAMESPACE::LoggerJniCallback>(
-      new ROCKSDB_NAMESPACE::LoggerJniCallback(env, jobj));
-
-  // set log level
-  auto* db_options =
-      reinterpret_cast<ROCKSDB_NAMESPACE::DBOptions*>(jdb_options);
-  sptr_logger->get()->SetInfoLogLevel(db_options->info_log_level);
-
+  auto log_level = static_cast<ROCKSDB_NAMESPACE::InfoLogLevel>(jlog_level);
+  sptr_logger->get()->SetInfoLogLevel(log_level);
   return GET_CPLUSPLUS_POINTER(sptr_logger);
 }
 
