@@ -4,26 +4,28 @@
 //  (found in the LICENSE.Apache file in the root directory).
 //
 // This file implements the "bridge" between Java and C++ and enables
-// calling C++ rocksdb::RestoreOptions methods
+// calling C++ ROCKSDB_NAMESPACE::RestoreOptions methods
 // from Java side.
 
+#include <jni.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <jni.h>
+
 #include <string>
 
 #include "include/org_rocksdb_RestoreOptions.h"
+#include "rocksdb/utilities/backup_engine.h"
+#include "rocksjni/cplusplus_to_java_convert.h"
 #include "rocksjni/portal.h"
-#include "rocksdb/utilities/backupable_db.h"
 /*
  * Class:     org_rocksdb_RestoreOptions
  * Method:    newRestoreOptions
  * Signature: (Z)J
  */
-jlong Java_org_rocksdb_RestoreOptions_newRestoreOptions(JNIEnv* env,
-    jclass jcls, jboolean keep_log_files) {
-  auto* ropt = new rocksdb::RestoreOptions(keep_log_files);
-  return reinterpret_cast<jlong>(ropt);
+jlong Java_org_rocksdb_RestoreOptions_newRestoreOptions(
+    JNIEnv* /*env*/, jclass /*jcls*/, jboolean keep_log_files) {
+  auto* ropt = new ROCKSDB_NAMESPACE::RestoreOptions(keep_log_files);
+  return GET_CPLUSPLUS_POINTER(ropt);
 }
 
 /*
@@ -31,9 +33,10 @@ jlong Java_org_rocksdb_RestoreOptions_newRestoreOptions(JNIEnv* env,
  * Method:    disposeInternal
  * Signature: (J)V
  */
-void Java_org_rocksdb_RestoreOptions_disposeInternal(JNIEnv* env, jobject jobj,
-    jlong jhandle) {
-  auto* ropt = reinterpret_cast<rocksdb::RestoreOptions*>(jhandle);
+void Java_org_rocksdb_RestoreOptions_disposeInternalJni(JNIEnv* /*env*/,
+                                                        jclass /*jobj*/,
+                                                        jlong jhandle) {
+  auto* ropt = reinterpret_cast<ROCKSDB_NAMESPACE::RestoreOptions*>(jhandle);
   assert(ropt);
   delete ropt;
 }
