@@ -21,5 +21,10 @@ public class CassandraValueMergeOperator extends MergeOperator {
   private static native long newSharedCassandraValueMergeOperator(
       int gcGracePeriodInSeconds, int limit);
 
-  @Override protected final native void disposeInternal(final long handle);
+  @Override
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+
+  private static native void disposeInternalJni(final long handle);
 }
