@@ -35,10 +35,7 @@ package org.rocksdb;
  * {@link org.rocksdb.InfoLogLevel#FATAL_LEVEL}.
  * </p>
  */
-public abstract class Logger extends RocksCallbackObject {
-  private static final long WITH_OPTIONS = 0;
-  private static final long WITH_DBOPTIONS = 1;
-
+public abstract class Logger extends RocksCallbackObject implements LoggerInterface {
   /**
    * <p>AbstractLogger constructor.</p>
    *
@@ -47,10 +44,13 @@ public abstract class Logger extends RocksCallbackObject {
    * maximum log level of RocksDB.</p>
    *
    * @param options {@link org.rocksdb.Options} instance.
+   *
+   * @deprecated Use {@link Logger#Logger(InfoLogLevel)} instead, e.g. {@code new
+   *     Logger(options.infoLogLevel())}.
    */
+  @Deprecated
   public Logger(final Options options) {
-    super(options.nativeHandle_, WITH_OPTIONS);
-
+    this(options.infoLogLevel());
   }
 
   /**
@@ -61,56 +61,64 @@ public abstract class Logger extends RocksCallbackObject {
    * as maximum log level of RocksDB.</p>
    *
    * @param dboptions {@link org.rocksdb.DBOptions} instance.
+   *
+   * @deprecated Use {@link Logger#Logger(InfoLogLevel)} instead, e.g. {@code new
+   *     Logger(dbOptions.infoLogLevel())}.
    */
+  @Deprecated
   public Logger(final DBOptions dboptions) {
-    super(dboptions.nativeHandle_, WITH_DBOPTIONS);
+    this(dboptions.infoLogLevel());
+  }
+
+  /**
+   * <p>AbstractLogger constructor.</p>
+   *
+   * @param logLevel the log level.
+   */
+  public Logger(final InfoLogLevel logLevel) {
+    super(logLevel.getValue());
   }
 
   @Override
   protected long initializeNative(final long... nativeParameterHandles) {
-    if(nativeParameterHandles[1] == WITH_OPTIONS) {
-      return createNewLoggerOptions(nativeParameterHandles[0]);
-    } else if(nativeParameterHandles[1] == WITH_DBOPTIONS) {
-      return createNewLoggerDbOptions(nativeParameterHandles[0]);
+    if (nativeParameterHandles.length == 1) {
+      return newLogger(nativeParameterHandles[0]);
     } else {
       throw new IllegalArgumentException();
     }
   }
 
-  /**
-   * Set {@link org.rocksdb.InfoLogLevel} to AbstractLogger.
-   *
-   * @param infoLogLevel {@link org.rocksdb.InfoLogLevel} instance.
-   */
-  public void setInfoLogLevel(final InfoLogLevel infoLogLevel) {
-      setInfoLogLevel(nativeHandle_, infoLogLevel.getValue());
+  @Override
+  public void setInfoLogLevel(final InfoLogLevel logLevel) {
+    setInfoLogLevel(nativeHandle_, logLevel.getValue());
   }
 
-  /**
-   * Return the loggers log level.
-   *
-   * @return {@link org.rocksdb.InfoLogLevel} instance.
-   */
+  @Override
   public InfoLogLevel infoLogLevel() {
     return InfoLogLevel.getInfoLogLevel(
         infoLogLevel(nativeHandle_));
   }
 
-  protected abstract void log(InfoLogLevel infoLogLevel,
-      String logMsg);
+  @Override
+  public long getNativeHandle() {
+    return nativeHandle_;
+  }
 
-  protected native long createNewLoggerOptions(
-      long options);
-  protected native long createNewLoggerDbOptions(
-      long dbOptions);
-  protected native void setInfoLogLevel(long handle,
-      byte infoLogLevel);
-  protected native byte infoLogLevel(long handle);
+  @Override
+  public final LoggerType getLoggerType() {
+    return LoggerType.JAVA_IMPLEMENTATION;
+  }
+
+  protected abstract void log(final InfoLogLevel logLevel, final String logMsg);
+
+  protected native long newLogger(final long logLevel);
+  protected native void setInfoLogLevel(final long handle, final byte logLevel);
+  protected native byte infoLogLevel(final long handle);
 
   /**
    * We override {@link RocksCallbackObject#disposeInternal()}
    * as disposing of a rocksdb::LoggerJniCallback requires
-   * a slightly different approach as it is a std::shared_ptr
+   * a slightly different approach as it is a std::shared_ptr.
    */
   @Override
   protected void disposeInternal() {
