@@ -44,10 +44,12 @@ public class OptimisticTransactionOptions extends RocksObject
   }
 
   private static native long newOptimisticTransactionOptions();
-  private native boolean isSetSnapshot(final long handle);
-  private native void setSetSnapshot(final long handle,
-      final boolean setSnapshot);
-  private native void setComparator(final long handle,
-      final long comparatorHandle);
-  @Override protected final native void disposeInternal(final long handle);
+  private static native boolean isSetSnapshot(final long handle);
+  private static native void setSetSnapshot(final long handle, final boolean setSnapshot);
+  private static native void setComparator(final long handle, final long comparatorHandle);
+  @Override
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+  private static native void disposeInternalJni(final long handle);
 }
