@@ -28,5 +28,9 @@ public class RestoreOptions extends RocksObject {
   }
 
   private static native long newRestoreOptions(boolean keepLogFiles);
-  @Override protected final native void disposeInternal(final long handle);
+  @Override
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+  private static native void disposeInternalJni(final long handle);
 }
