@@ -25,5 +25,9 @@ public class StringAppendOperator extends MergeOperator {
 
   private static native long newSharedStringAppendOperator(final char delim);
   private static native long newSharedStringAppendOperator(final String delim);
-  @Override protected final native void disposeInternal(final long handle);
+  @Override
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+  private static native void disposeInternalJni(final long handle);
 }
