@@ -289,7 +289,7 @@ public class OptionsUtilTest {
     altCFTableConfig.setIndexBlockRestartInterval(6);
     altCFTableConfig.setMetadataBlockSize(12 * 1024);
     altCFTableConfig.setPartitionFilters(true);
-    altCFTableConfig.setOptimizeFiltersForMemory(true);
+    altCFTableConfig.setOptimizeFiltersForMemory(false);
     altCFTableConfig.setUseDeltaEncoding(false);
     altCFTableConfig.setFilterPolicy(new BloomFilter(7.5));
     altCFTableConfig.setWholeKeyFiltering(false);
@@ -358,6 +358,7 @@ public class OptionsUtilTest {
     assertThat(actual.metadataBlockSize()).isEqualTo(expected.metadataBlockSize());
     assertThat(actual.partitionFilters()).isEqualTo(expected.partitionFilters());
     assertThat(actual.optimizeFiltersForMemory()).isEqualTo(expected.optimizeFiltersForMemory());
+    assertThat(actual.useDeltaEncoding()).isEqualTo(expected.useDeltaEncoding());
     assertThat(actual.wholeKeyFiltering()).isEqualTo(expected.wholeKeyFiltering());
     assertThat(actual.verifyCompression()).isEqualTo(expected.verifyCompression());
     assertThat(actual.readAmpBytesPerBit()).isEqualTo(expected.readAmpBytesPerBit());
@@ -370,9 +371,5 @@ public class OptionsUtilTest {
     } else {
       assertThat(expected.filterPolicy().equals(actual.filterPolicy()));
     }
-
-    // not currently persisted - always true when read from options
-    // this test will fail, and need repaired, if and when "useDeltaEncoding" is persisted.
-    assertThat(actual.useDeltaEncoding()).isEqualTo(true);
   }
 }
