@@ -75,7 +75,7 @@ class EnvLogger : public Logger {
     mutex_.AssertHeld();
     if (flush_pending_) {
       flush_pending_ = false;
-      file_.Flush().PermitUncheckedError();
+      file_.Flush(IOOptions()).PermitUncheckedError();
       file_.reset_seen_error();
     }
     last_flush_micros_ = clock_->NowMicros();
@@ -93,7 +93,7 @@ class EnvLogger : public Logger {
 
   Status CloseHelper() {
     FileOpGuard guard(*this);
-    const auto close_status = file_.Close();
+    const auto close_status = file_.Close(IOOptions());
 
     if (close_status.ok()) {
       return close_status;
@@ -162,7 +162,7 @@ class EnvLogger : public Logger {
       {
         FileOpGuard guard(*this);
         // We will ignore any error returned by Append().
-        file_.Append(Slice(base, p - base)).PermitUncheckedError();
+        file_.Append(IOOptions(), Slice(base, p - base)).PermitUncheckedError();
         file_.reset_seen_error();
         flush_pending_ = true;
         const uint64_t now_micros = clock_->NowMicros();
