@@ -17,13 +17,25 @@
 
 namespace ROCKSDB_NAMESPACE {
 
+namespace {
+intptr_t GetOffset(const Configurable* holder, void* field) {
+  return reinterpret_cast<intptr_t>(field) -
+         reinterpret_cast<intptr_t>(static_cast<const void*>(holder));
+}
+
+void* ApplyOffset(const Configurable* holder, intptr_t offset) {
+  return reinterpret_cast<void*>(
+      reinterpret_cast<intptr_t>(static_cast<const void*>(holder)) + offset);
+}
+}  // namespace
+
 void Configurable::RegisterOptions(
     const std::string& name, void* opt_ptr,
     const std::unordered_map<std::string, OptionTypeInfo>* type_map) {
   RegisteredOptions opts;
   opts.name = name;
   opts.type_map = type_map;
-  opts.opt_ptr = opt_ptr;
+  opts.opt_offset = GetOffset(this, opt_ptr);
   options_.emplace_back(opts);
 }
 
@@ -37,12 +49,13 @@ Status Configurable::PrepareOptions(const ConfigOptions& opts) {
   // We ignore the invoke_prepare_options here intentionally,
   // as if you are here, you must have called PrepareOptions explicitly.
   Status status = Status::OK();
-  for (auto opt_iter : options_) {
+  for (const auto& opt_iter : options_) {
     if (opt_iter.type_map != nullptr) {
-      for (auto map_iter : *(opt_iter.type_map)) {
+      for (const auto& map_iter : *(opt_iter.type_map)) {
         auto& opt_info = map_iter.second;
         if (opt_info.ShouldPrepare()) {
-          status = opt_info.Prepare(opts, map_iter.first, opt_iter.opt_ptr);
+          status = opt_info.Prepare(opts, map_iter.first,
+                                    ApplyOffset(this, opt_iter.opt_offset));
           if (!status.ok()) {
             return status;
           }
@@ -56,13 +69,13 @@ Status Configurable::PrepareOptions(const ConfigOptions& opts) {
 Status Configurable::ValidateOptions(const DBOptions& db_opts,
                                      const ColumnFamilyOptions& cf_opts) const {
   Status status;
-  for (auto opt_iter : options_) {
+  for (const auto& opt_iter : options_) {
     if (opt_iter.type_map != nullptr) {
-      for (auto map_iter : *(opt_iter.type_map)) {
+      for (const auto& map_iter : *(opt_iter.type_map)) {
         auto& opt_info = map_iter.second;
         if (opt_info.ShouldValidate()) {
           status = opt_info.Validate(db_opts, cf_opts, map_iter.first,
-                                     opt_iter.opt_ptr);
+                                     ApplyOffset(this, opt_iter.opt_offset));
           if (!status.ok()) {
             return status;
           }
@@ -80,9 +93,9 @@ Status Configurable::ValidateOptions(const DBOptions& db_opts,
 /*********************************************************************************/
 
 const void* Configurable::GetOptionsPtr(const std::string& name) const {
-  for (auto o : options_) {
+  for (const auto& o : options_) {
     if (o.name == name) {
-      return o.opt_ptr;
+      return ApplyOffset(this, o.opt_offset);
     }
   }
   return nullptr;
@@ -93,14 +106,14 @@ std::string Configurable::GetOptionName(const std::string& opt_name) const {
 }
 
 const OptionTypeInfo* ConfigurableHelper::FindOption(
-    const std::vector<Configurable::RegisteredOptions>& options,
-    const std::string& short_name, std::string* opt_name, void** opt_ptr) {
-  for (auto iter : options) {
+    const Configurable& configurable, const std::string& short_name,
+    std::string* opt_name, void** opt_ptr) {
+  for (const auto& iter : configurable.options_) {
     if (iter.type_map != nullptr) {
       const auto opt_info =
           OptionTypeInfo::Find(short_name, *(iter.type_map), opt_name);
       if (opt_info != nullptr) {
-        *opt_ptr = iter.opt_ptr;
+        *opt_ptr = ApplyOffset(&configurable, iter.opt_offset);
         return opt_info;
       }
     }
@@ -244,7 +257,8 @@ Status ConfigurableHelper::ConfigureOptions(
     for (const auto& iter : configurable.options_) {
       if (iter.type_map != nullptr) {
         s = ConfigureSomeOptions(config_options, configurable, *(iter.type_map),
-                                 &remaining, iter.opt_ptr);
+                                 &remaining,
+                                 ApplyOffset(&configurable, iter.opt_offset));
         if (remaining.empty()) {  // Are there more options left?
           break;
         } else if (!s.ok()) {
@@ -318,21 +332,29 @@ Status ConfigurableHelper::ConfigureSomeOptions(
   }    // End while found one or options remain
 
   // Now that we have been through the list, remove any unsupported
-  for (auto u : unsupported) {
+  for (const auto& u : unsupported) {
     auto it = options->find(u);
     if (it != options->end()) {
       options->erase(it);
     }
   }
   if (config_options.ignore_unknown_options) {
-    if (!result.ok()) result.PermitUncheckedError();
-    if (!notsup.ok()) notsup.PermitUncheckedError();
+    if (!result.ok()) {
+      result.PermitUncheckedError();
+    }
+    if (!notsup.ok()) {
+      notsup.PermitUncheckedError();
+    }
     return Status::OK();
   } else if (!result.ok()) {
-    if (!notsup.ok()) notsup.PermitUncheckedError();
+    if (!notsup.ok()) {
+      notsup.PermitUncheckedError();
+    }
     return result;
   } else if (config_options.ignore_unsupported_options) {
-    if (!notsup.ok()) notsup.PermitUncheckedError();
+    if (!notsup.ok()) {
+      notsup.PermitUncheckedError();
+    }
     return Status::OK();
   } else {
     return notsup;
@@ -346,7 +368,7 @@ Status ConfigurableHelper::ConfigureSingleOption(
   std::string elem_name;
   void* opt_ptr = nullptr;
   const auto opt_info =
-      FindOption(configurable.options_, opt_name, &elem_name, &opt_ptr);
+      FindOption(configurable, opt_name, &elem_name, &opt_ptr);
   if (opt_info == nullptr) {
     return Status::NotFound("Could not find option: ", name);
   } else {
@@ -374,7 +396,7 @@ Status ConfigurableHelper::ConfigureCustomizableOption(
       return Status::OK();
     } else if (custom == nullptr || !StartsWith(name, custom->GetId() + ".")) {
       return configurable.ParseOption(copy, opt_info, name, value, opt_ptr);
-    } else if (value.find("=") != std::string::npos) {
+    } else if (value.find('=') != std::string::npos) {
       return custom->ConfigureFromString(copy, value);
     } else {
       return custom->ConfigureOption(copy, name, value);
@@ -499,7 +521,7 @@ Status ConfigurableHelper::GetOption(const ConfigOptions& config_options,
   std::string opt_name;
   void* opt_ptr = nullptr;
   const auto opt_info =
-      FindOption(configurable.options_, short_name, &opt_name, &opt_ptr);
+      FindOption(configurable, short_name, &opt_name, &opt_ptr);
   if (opt_info != nullptr) {
     ConfigOptions embedded = config_options;
     embedded.delimiter = ";";
@@ -530,22 +552,22 @@ Status ConfigurableHelper::SerializeOptions(const ConfigOptions& config_options,
         if (opt_info.ShouldSerialize()) {
           std::string value;
           Status s;
+          void* opt_ptr = ApplyOffset(&configurable, opt_iter.opt_offset);
           if (!config_options.mutable_options_only) {
-            s = opt_info.Serialize(config_options, prefix + opt_name,
-                                   opt_iter.opt_ptr, &value);
+            s = opt_info.Serialize(config_options, prefix + opt_name, opt_ptr,
+                                   &value);
           } else if (opt_info.IsMutable()) {
             ConfigOptions copy = config_options;
             copy.mutable_options_only = false;
-            s = opt_info.Serialize(copy, prefix + opt_name, opt_iter.opt_ptr,
-                                   &value);
+            s = opt_info.Serialize(copy, prefix + opt_name, opt_ptr, &value);
           } else if (opt_info.IsConfigurable()) {
             // If it is a Configurable and we are either printing all of the
             // details or not printing only the name, this option should be
             // included in the list
             if (config_options.IsDetailed() ||
                 !opt_info.IsEnabled(OptionTypeFlags::kStringNameOnly)) {
-              s = opt_info.Serialize(config_options, prefix + opt_name,
-                                     opt_iter.opt_ptr, &value);
+              s = opt_info.Serialize(config_options, prefix + opt_name, opt_ptr,
+                                     &value);
             }
           }
           if (!s.ok()) {
