@@ -32,23 +32,30 @@ class PartitionedFilterBlockBuilder : public FullFilterBlockBuilder {
       const bool use_value_delta_encoding,
       PartitionedIndexBuilder* const p_index_builder,
       const uint32_t partition_size, size_t ts_sz,
-      const bool persist_user_defined_timestamps);
+      const bool persist_user_defined_timestamps,
+      bool decouple_from_index_partitions);
 
   virtual ~PartitionedFilterBlockBuilder();
 
-  void AddKey(const Slice& key) override;
-  void Add(const Slice& key) override;
+  void Add(const Slice& key_without_ts) override;
+  void AddWithPrevKey(const Slice& key_without_ts,
+                      const Slice& prev_key_without_ts) override;
+  bool IsEmpty() const override {
+    return filter_bits_builder_->EstimateEntriesAdded() == 0 &&
+           filters_.empty();
+  }
+
   size_t EstimateEntriesAdded() override;
 
-  virtual Slice Finish(
-      const BlockHandle& last_partition_block_handle, Status* status,
-      std::unique_ptr<const char[]>* filter_data = nullptr) override;
+  void PrevKeyBeforeFinish(const Slice& prev_key_without_ts) override;
+  Status Finish(const BlockHandle& last_partition_block_handle, Slice* filter,
+                std::unique_ptr<const char[]>* filter_owner = nullptr) override;
 
-  virtual void ResetFilterBitsBuilder() override {
-    // Previously constructed partitioned filters by
-    // this to-be-reset FiterBitsBuilder can also be
-    // cleared
-    filters.clear();
+  void ResetFilterBitsBuilder() override {
+    filters_.clear();
+    total_added_in_built_ = 0;
+    index_on_filter_block_builder_.Reset();
+    index_on_filter_block_builder_without_seq_.Reset();
     FullFilterBlockBuilder::ResetFilterBitsBuilder();
   }
 
@@ -56,49 +63,69 @@ class PartitionedFilterBlockBuilder : public FullFilterBlockBuilder {
   // as part of PartitionFilterBlockBuilder::Finish
   // to avoid implementation complexity of doing it elsewhere.
   // Therefore we are skipping it in here.
-  virtual Status MaybePostVerifyFilter(
-      const Slice& /* filter_content */) override {
+  Status MaybePostVerifyFilter(const Slice& /* filter_content */) override {
     return Status::OK();
   }
 
- private:
+ private:  // fns
+  // Whether to cut a filter block before the next key
+  bool DecideCutAFilterBlock();
+  void CutAFilterBlock(const Slice* next_key, const Slice* next_prefix,
+                       const Slice& prev_key);
+
+  void AddImpl(const Slice& key_without_ts, const Slice& prev_key_without_ts);
+
+ private:  // data
+  // Currently we keep the same number of partitions for filters and indexes.
+  // This would allow for some potentioal optimizations in future. If such
+  // optimizations did not realize we can use different number of partitions and
+  // eliminate p_index_builder_
+  PartitionedIndexBuilder* const p_index_builder_;
+  const size_t ts_sz_;
+  const bool decouple_from_index_partitions_;
+
   // Filter data
-  BlockBuilder index_on_filter_block_builder_;  // top-level index builder
-  BlockBuilder
-      index_on_filter_block_builder_without_seq_;  // same for user keys
   struct FilterEntry {
-    std::string key;
-    std::unique_ptr<const char[]> filter_data;
+    std::string ikey;  // internal key or separator *after* this filter
+    std::unique_ptr<const char[]> filter_owner;
     Slice filter;
   };
-  std::deque<FilterEntry> filters;  // list of partitioned filters and keys used
-                                    // in building the index
+  std::deque<FilterEntry> filters_;  // list of partitioned filters and keys
+                                     // used in building the index
+  // The desired number of keys per partition
+  uint32_t keys_per_partition_;
+  // According to the bits builders, how many keys/prefixes added
+  // in all the filters we have fully built
+  uint64_t total_added_in_built_ = 0;
 
   // Set to the first non-okay status if any of the filter
   // partitions experiences construction error.
   // If partitioned_filters_construction_status_ is non-okay,
   // then the whole partitioned filters should not be used.
   Status partitioned_filters_construction_status_;
-  std::string last_filter_entry_key;
-  std::unique_ptr<const char[]> last_filter_data;
-  std::unique_ptr<IndexBuilder> value;
-  bool finishing_filters =
-      false;  // true if Finish is called once but not complete yet.
-  // The policy of when cut a filter block and Finish it
-  void MaybeCutAFilterBlock(const Slice* next_key);
-  // Currently we keep the same number of partitions for filters and indexes.
-  // This would allow for some potentioal optimizations in future. If such
-  // optimizations did not realize we can use different number of partitions and
-  // eliminate p_index_builder_
-  PartitionedIndexBuilder* const p_index_builder_;
-  // The desired number of keys per partition
-  uint32_t keys_per_partition_;
-  // The number of keys added to the last partition so far
-  uint32_t keys_added_to_partition_;
-  // According to the bits builders, how many keys/prefixes added
-  // in all the filters we have fully built
-  uint64_t total_added_in_built_;
+
+  // For Add without prev key
+  std::string prev_key_without_ts_;
+
+#ifndef NDEBUG
+  // For verifying accurate previous keys are provided by the caller, so that
+  // release code can be fast
+  bool DEBUG_add_with_prev_key_called_ = false;
+  std::string DEBUG_prev_key_without_ts_;
+#endif  // NDEBUG
+
+  // ===== State for Finish() =====
+
+  // top-level index builder on internal keys
+  BlockBuilder index_on_filter_block_builder_;
+  // same for user keys
+  BlockBuilder index_on_filter_block_builder_without_seq_;
+  // For delta-encoding handles
   BlockHandle last_encoded_handle_;
+  // True if we are between two calls to Finish(), because we have returned
+  // the filter at the front of filters_ but haven't yet added it to the
+  // partition index.
+  bool finishing_front_filter_ = false;
 };
 
 class PartitionedFilterBlockReader
@@ -113,22 +140,20 @@ class PartitionedFilterBlockReader
       FilePrefetchBuffer* prefetch_buffer, bool use_cache, bool prefetch,
       bool pin, BlockCacheLookupContext* lookup_context);
 
-  bool KeyMayMatch(const Slice& key, const bool no_io,
-                   const Slice* const const_ikey_ptr, GetContext* get_context,
+  bool KeyMayMatch(const Slice& key, const Slice* const const_ikey_ptr,
+                   GetContext* get_context,
                    BlockCacheLookupContext* lookup_context,
                    const ReadOptions& read_options) override;
-  void KeysMayMatch(MultiGetRange* range, const bool no_io,
+  void KeysMayMatch(MultiGetRange* range,
                     BlockCacheLookupContext* lookup_context,
                     const ReadOptions& read_options) override;
 
-  bool PrefixMayMatch(const Slice& prefix, const bool no_io,
-                      const Slice* const const_ikey_ptr,
+  bool PrefixMayMatch(const Slice& prefix, const Slice* const const_ikey_ptr,
                       GetContext* get_context,
                       BlockCacheLookupContext* lookup_context,
                       const ReadOptions& read_options) override;
   void PrefixesMayMatch(MultiGetRange* range,
                         const SliceTransform* prefix_extractor,
-                        const bool no_io,
                         BlockCacheLookupContext* lookup_context,
                         const ReadOptions& read_options) override;
 
@@ -140,35 +165,36 @@ class PartitionedFilterBlockReader
       const Slice& entry) const;
   Status GetFilterPartitionBlock(
       FilePrefetchBuffer* prefetch_buffer, const BlockHandle& handle,
-      bool no_io, GetContext* get_context,
-      BlockCacheLookupContext* lookup_context, const ReadOptions& read_options,
+      GetContext* get_context, BlockCacheLookupContext* lookup_context,
+      const ReadOptions& read_options,
       CachableEntry<ParsedFullFilterBlock>* filter_block) const;
 
   using FilterFunction = bool (FullFilterBlockReader::*)(
-      const Slice& slice, const bool no_io, const Slice* const const_ikey_ptr,
+      const Slice& slice, const Slice* const const_ikey_ptr,
       GetContext* get_context, BlockCacheLookupContext* lookup_context,
       const ReadOptions& read_options);
-  bool MayMatch(const Slice& slice, bool no_io, const Slice* const_ikey_ptr,
+  bool MayMatch(const Slice& slice, const Slice* const_ikey_ptr,
                 GetContext* get_context,
                 BlockCacheLookupContext* lookup_context,
                 const ReadOptions& read_options,
                 FilterFunction filter_function) const;
   using FilterManyFunction = void (FullFilterBlockReader::*)(
       MultiGetRange* range, const SliceTransform* prefix_extractor,
-      const bool no_io, BlockCacheLookupContext* lookup_context,
-      const ReadOptions& read_options);
+      BlockCacheLookupContext* lookup_context, const ReadOptions& read_options);
   void MayMatch(MultiGetRange* range, const SliceTransform* prefix_extractor,
-                bool no_io, BlockCacheLookupContext* lookup_context,
+                BlockCacheLookupContext* lookup_context,
                 const ReadOptions& read_options,
                 FilterManyFunction filter_function) const;
   void MayMatchPartition(MultiGetRange* range,
                          const SliceTransform* prefix_extractor,
-                         BlockHandle filter_handle, bool no_io,
+                         BlockHandle filter_handle,
                          BlockCacheLookupContext* lookup_context,
                          const ReadOptions& read_options,
                          FilterManyFunction filter_function) const;
   Status CacheDependencies(const ReadOptions& ro, bool pin,
                            FilePrefetchBuffer* tail_prefetch_buffer) override;
+  void EraseFromCacheBeforeDestruction(
+      uint32_t /*uncache_aggressiveness*/) override;
 
   const InternalKeyComparator* internal_comparator() const;
   bool index_key_includes_seq() const;
