@@ -4,12 +4,13 @@
 //  (found in the LICENSE.Apache file in the root directory).
 
 #pragma once
-#ifndef ROCKSDB_LITE
 #include <stdint.h>
+
 #include <limits>
 #include <string>
 #include <utility>
 #include <vector>
+
 #include "db/version_edit.h"
 #include "port/port.h"
 #include "rocksdb/status.h"
@@ -20,7 +21,7 @@
 
 namespace ROCKSDB_NAMESPACE {
 
-class CuckooTableBuilder: public TableBuilder {
+class CuckooTableBuilder : public TableBuilder {
  public:
   CuckooTableBuilder(
       WritableFileWriter* file, double max_hash_table_ratio,
@@ -78,8 +79,7 @@ class CuckooTableBuilder: public TableBuilder {
 
  private:
   struct CuckooBucket {
-    CuckooBucket()
-      : vector_idx(kMaxVectorIdx), make_space_for_key_call_id(0) {}
+    CuckooBucket() : vector_idx(kMaxVectorIdx), make_space_for_key_call_id(0) {}
     uint32_t vector_idx;
     // This number will not exceed kvs_.size() + max_num_hash_func_.
     // We assume number of items is <= 2^32.
@@ -125,7 +125,7 @@ class CuckooTableBuilder: public TableBuilder {
   bool use_module_hash_;
   bool identity_as_first_hash_;
   uint64_t (*get_slice_hash_)(const Slice& s, uint32_t index,
-    uint64_t max_num_buckets);
+                              uint64_t max_num_buckets);
   std::string largest_user_key_ = "";
   std::string smallest_user_key_ = "";
 
@@ -134,4 +134,3 @@ class CuckooTableBuilder: public TableBuilder {
 
 }  // namespace ROCKSDB_NAMESPACE
 
-#endif  // ROCKSDB_LITE
