@@ -52,12 +52,12 @@ IOStatus WriteBlock(const Slice& block_contents, WritableFileWriter* file,
 // kPlainTableMagicNumber was picked by running
 //    echo rocksdb.table.plain | sha1sum
 // and taking the leading 64 bits.
-extern const uint64_t kPlainTableMagicNumber = 0x8242229663bf9564ull;
-extern const uint64_t kLegacyPlainTableMagicNumber = 0x4f3418eb7a8f13b8ull;
+const uint64_t kPlainTableMagicNumber = 0x8242229663bf9564ull;
+const uint64_t kLegacyPlainTableMagicNumber = 0x4f3418eb7a8f13b8ull;
 
 PlainTableBuilder::PlainTableBuilder(
     const ImmutableOptions& ioptions, const MutableCFOptions& moptions,
-    const IntTblPropCollectorFactories* int_tbl_prop_collector_factories,
+    const InternalTblPropCollFactories* internal_tbl_prop_coll_factories,
     uint32_t column_family_id, int level_at_creation, WritableFileWriter* file,
     uint32_t user_key_len, EncodingType encoding_type, size_t index_sparseness,
     uint32_t bloom_bits_per_key, const std::string& column_family_name,
@@ -114,13 +114,13 @@ PlainTableBuilder::PlainTableBuilder(
   properties_
       .user_collected_properties[PlainTablePropertyNames::kEncodingType] = val;
 
-  assert(int_tbl_prop_collector_factories);
-  for (auto& factory : *int_tbl_prop_collector_factories) {
+  assert(internal_tbl_prop_coll_factories);
+  for (auto& factory : *internal_tbl_prop_coll_factories) {
     assert(factory);
 
-    std::unique_ptr<IntTblPropCollector> collector{
-        factory->CreateIntTblPropCollector(column_family_id,
-                                           level_at_creation)};
+    std::unique_ptr<InternalTblPropColl> collector{
+        factory->CreateInternalTblPropColl(column_family_id, level_at_creation,
+                                           ioptions.num_levels)};
     if (collector) {
       table_properties_collectors_.emplace_back(std::move(collector));
     }
