@@ -5,10 +5,10 @@
 
 #pragma once
 
-#ifndef ROCKSDB_LITE
+#include <stdint.h>
+
 #include <memory>
 #include <string>
-#include <stdint.h>
 
 #include "rocksdb/table.h"
 
@@ -156,6 +156,8 @@ class PlainTableFactory : public TableFactory {
   explicit PlainTableFactory(
       const PlainTableOptions& _table_options = PlainTableOptions());
 
+  // Method to allow CheckedCast to work for this class
+  static const char* kClassName() { return kPlainTableName(); }
   const char* Name() const override { return kPlainTableName(); }
   using TableFactory::NewTableReader;
   Status NewTableReader(const ReadOptions& ro,
@@ -166,15 +168,17 @@ class PlainTableFactory : public TableFactory {
 
   TableBuilder* NewTableBuilder(
       const TableBuilderOptions& table_builder_options,
-      uint32_t column_family_id, WritableFileWriter* file) const override;
+      WritableFileWriter* file) const override;
 
   std::string GetPrintableOptions() const override;
   static const char kValueTypeSeqId0 = char(~0);
 
+  std::unique_ptr<TableFactory> Clone() const override {
+    return std::make_unique<PlainTableFactory>(*this);
+  }
+
  private:
   PlainTableOptions table_options_;
 };
 
-
 }  // namespace ROCKSDB_NAMESPACE
-#endif  // ROCKSDB_LITE
