@@ -4,7 +4,6 @@
 //  (found in the LICENSE.Apache file in the root directory).
 
 #pragma once
-#ifndef ROCKSDB_LITE
 
 #include "rocksdb/utilities/backup_engine.h"
 
@@ -23,7 +22,7 @@ struct TEST_BackupMetaSchemaOptions {
 // unpublished schema version 2, for the life of this object (not backup_dir).
 // TEST_BackupMetaSchemaOptions offers some customization for testing.
 void TEST_SetBackupMetaSchemaOptions(
-    BackupEngine *engine, const TEST_BackupMetaSchemaOptions &options);
+    BackupEngine* engine, const TEST_BackupMetaSchemaOptions& options);
 
 // Modifies the BackupEngine(Impl) to use specified clocks for backup and
 // restore rate limiters created by default if not specified by users for
@@ -33,4 +32,3 @@ void TEST_SetDefaultRateLimitersClock(
     const std::shared_ptr<SystemClock>& backup_rate_limiter_clock = nullptr,
     const std::shared_ptr<SystemClock>& restore_rate_limiter_clock = nullptr);
 }  // namespace ROCKSDB_NAMESPACE
-#endif  // ROCKSDB_LITE
