@@ -4,9 +4,9 @@
 //  COPYING file in the root directory) and Apache 2.0 License
 //  (found in the LICENSE.Apache file in the root directory).
 
-#ifndef ROCKSDB_LITE
 
 #include "rocksdb/utilities/env_mirror.h"
+
 #include "env/mock_env.h"
 #include "test_util/testharness.h"
 
@@ -15,7 +15,7 @@ namespace ROCKSDB_NAMESPACE {
 class EnvMirrorTest : public testing::Test {
  public:
   Env* default_;
-  MockEnv* a_, *b_;
+  MockEnv *a_, *b_;
   EnvMirror* env_;
   const EnvOptions soptions_;
 
@@ -97,8 +97,9 @@ TEST_F(EnvMirrorTest, Basics) {
   ASSERT_TRUE(
       !env_->NewSequentialFile("/dir/non_existent", &seq_file, soptions_).ok());
   ASSERT_TRUE(!seq_file);
-  ASSERT_TRUE(!env_->NewRandomAccessFile("/dir/non_existent", &rand_file,
-                                         soptions_).ok());
+  ASSERT_TRUE(
+      !env_->NewRandomAccessFile("/dir/non_existent", &rand_file, soptions_)
+           .ok());
   ASSERT_TRUE(!rand_file);
 
   // Check that deleting works.
@@ -213,12 +214,3 @@ int main(int argc, char** argv) {
   return RUN_ALL_TESTS();
 }
 
-#else
-#include <stdio.h>
-
-int main(int argc, char** argv) {
-  fprintf(stderr, "SKIPPED as EnvMirror is not supported in ROCKSDB_LITE\n");
-  return 0;
-}
-
-#endif  // !ROCKSDB_LITE
