@@ -72,30 +72,25 @@ class TestWritableFile : public WritableFile {
                             std::unique_ptr<WritableFile>&& f,
                             FaultInjectionTestEnv* env);
   virtual ~TestWritableFile();
-  virtual Status Append(const Slice& data) override;
-  virtual Status Append(
-      const Slice& data,
-      const DataVerificationInfo& /*verification_info*/) override {
+  Status Append(const Slice& data) override;
+  Status Append(const Slice& data,
+                const DataVerificationInfo& /*verification_info*/) override {
     return Append(data);
   }
-  virtual Status Truncate(uint64_t size) override {
-    return target_->Truncate(size);
-  }
-  virtual Status Close() override;
-  virtual Status Flush() override;
-  virtual Status Sync() override;
-  virtual bool IsSyncThreadSafe() const override { return true; }
-  virtual Status PositionedAppend(const Slice& data, uint64_t offset) override {
+  Status Truncate(uint64_t size) override { return target_->Truncate(size); }
+  Status Close() override;
+  Status Flush() override;
+  Status Sync() override;
+  bool IsSyncThreadSafe() const override { return true; }
+  Status PositionedAppend(const Slice& data, uint64_t offset) override {
     return target_->PositionedAppend(data, offset);
   }
-  virtual Status PositionedAppend(
+  Status PositionedAppend(
       const Slice& data, uint64_t offset,
       const DataVerificationInfo& /*verification_info*/) override {
     return PositionedAppend(data, offset);
   }
-  virtual bool use_direct_io() const override {
-    return target_->use_direct_io();
-  };
+  bool use_direct_io() const override { return target_->use_direct_io(); }
   uint64_t GetFileSize() final { return target_->GetFileSize(); }
 
  private:
@@ -122,7 +117,7 @@ class TestRandomRWFile : public RandomRWFile {
   size_t GetRequiredBufferAlignment() const override {
     return target_->GetRequiredBufferAlignment();
   }
-  bool use_direct_io() const override { return target_->use_direct_io(); };
+  bool use_direct_io() const override { return target_->use_direct_io(); }
 
  private:
   std::unique_ptr<RandomRWFile> target_;
@@ -137,8 +132,8 @@ class TestDirectory : public Directory {
       : env_(env), dirname_(dirname), dir_(dir) {}
   ~TestDirectory() {}
 
-  virtual Status Fsync() override;
-  virtual Status Close() override;
+  Status Fsync() override;
+  Status Close() override;
 
  private:
   FaultInjectionTestEnv* env_;
@@ -174,17 +169,15 @@ class FaultInjectionTestEnv : public EnvWrapper {
                              std::unique_ptr<RandomAccessFile>* result,
                              const EnvOptions& soptions) override;
 
-  virtual Status DeleteFile(const std::string& f) override;
+  Status DeleteFile(const std::string& f) override;
 
-  virtual Status RenameFile(const std::string& s,
-                            const std::string& t) override;
+  Status RenameFile(const std::string& s, const std::string& t) override;
 
-  virtual Status LinkFile(const std::string& s, const std::string& t) override;
+  Status LinkFile(const std::string& s, const std::string& t) override;
 
 // Undef to eliminate clash on Windows
 #undef GetFreeSpace
-  virtual Status GetFreeSpace(const std::string& path,
-                              uint64_t* disk_free) override {
+  Status GetFreeSpace(const std::string& path, uint64_t* disk_free) override {
     if (!IsFilesystemActive() &&
         error_.subcode() == IOStatus::SubCode::kNoSpace) {
       *disk_free = 0;
