@@ -17,8 +17,8 @@ namespace ROCKSDB_NAMESPACE {
 
 class TransactionDBMutexImpl : public TransactionDBMutex {
  public:
-  TransactionDBMutexImpl() {}
-  ~TransactionDBMutexImpl() override {}
+  TransactionDBMutexImpl() = default;
+  ~TransactionDBMutexImpl() override = default;
 
   Status Lock() override;
 
@@ -34,8 +34,8 @@ class TransactionDBMutexImpl : public TransactionDBMutex {
 
 class TransactionDBCondVarImpl : public TransactionDBCondVar {
  public:
-  TransactionDBCondVarImpl() {}
-  ~TransactionDBCondVarImpl() override {}
+  TransactionDBCondVarImpl() = default;
+  ~TransactionDBCondVarImpl() override = default;
 
   Status Wait(std::shared_ptr<TransactionDBMutex> mutex) override;
 
@@ -91,7 +91,7 @@ Status TransactionDBMutexImpl::TryLockFor(int64_t timeout_time) {
 
 Status TransactionDBCondVarImpl::Wait(
     std::shared_ptr<TransactionDBMutex> mutex) {
-  auto mutex_impl = reinterpret_cast<TransactionDBMutexImpl*>(mutex.get());
+  auto mutex_impl = static_cast<TransactionDBMutexImpl*>(mutex.get());
 
   std::unique_lock<std::mutex> lock(mutex_impl->mutex_, std::adopt_lock);
   cv_.wait(lock);
@@ -106,7 +106,7 @@ Status TransactionDBCondVarImpl::WaitFor(
     std::shared_ptr<TransactionDBMutex> mutex, int64_t timeout_time) {
   Status s;
 
-  auto mutex_impl = reinterpret_cast<TransactionDBMutexImpl*>(mutex.get());
+  auto mutex_impl = static_cast<TransactionDBMutexImpl*>(mutex.get());
   std::unique_lock<std::mutex> lock(mutex_impl->mutex_, std::adopt_lock);
 
   if (timeout_time < 0) {
